package webwisdom.pim;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;
import java.util.*;
import webwisdom.tango.pim.*;
import java.sql.*;

public class PIMServer extends UnicastRemoteObject implements PIMLogin
{
  public PIMServer() throws java.rmi.RemoteException
    {
    }
  public PIM authenticate(String login,String password) throws java.rmi.RemoteException
    {
      try
	{
	  PIMTools pt=new PIMTools();
	  if(pt.authenticate(login,password))
	    {
	      System.out.println("Returning interface");
	      return pt;
	    }
	  else
	    {
	      System.out.println("User unknown");
	      pt.dispose();
	      return null;
	    }
	}
      catch(Exception e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public static void main(String args[])
    {
      try
	{
	  System.setSecurityManager(new RMISecurityManager());
	  PIMServer ps=new PIMServer();
	  String localname=java.net.InetAddress.getLocalHost().getHostName();
	  
	  Naming.rebind("//"+localname+"/TangoPIMLogin",ps);
	  System.out.println("TangoPIM bound in registry");
	}
      catch(Exception e)
	{
	  System.out.println("PIMServer error:");
	  e.printStackTrace();
	  System.exit(1);
	}
	  
    }
}

class PIMTools extends UnicastRemoteObject implements PIM
{
  Connection conn;
  PreparedStatement pstmt1;
  PreparedStatement pstmt2;
  PreparedStatement pstmt_ru_set1;
  PreparedStatement pstmt_ru_set2;
  PreparedStatement pstmt_rc_set1;
  PreparedStatement pstmt_rc_set2;
  PreparedStatement pstmt_rg_set1;
  PreparedStatement pstmt_rg_set2;
  Random 	    random=new Random();
  int 		    uid;

  public void dispose() throws Exception
    {
      conn.close();
    }
  boolean authenticate(String login,String password) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt=conn.createStatement();
	  ResultSet rset=stmt.executeQuery("SELECT u_id FROM users WHERE ulogin='"+login+
					   "' AND upass='"+password+"'");
	  if(rset.next())
	    {
	      uid=rset.getInt(1);
	      return true;
	    }
	  else
	    return false;
	}
      catch(SQLException e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }

  public int getUserId()
    {
      return uid;
    }

  public int[] getCommunityUsers(int cid) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt=conn.createStatement();
	  ResultSet rset=stmt.executeQuery("SELECT uid FROM community_access WHERE cid="+cid);
	  Vector v=new Vector();
	  while(rset.next())
	    {
	      v.addElement(new Integer(rset.getInt(1)));
	    }
	  int table[]=new int[v.size()];
	  for(int i=0;i<v.size();i++)
	    table[i]=((Integer)v.elementAt(i)).intValue();
	  return table;
	}
      catch(Exception e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public Community[]	getAllowedCommunities(int uid) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt=conn.createStatement();
	  ResultSet rset=stmt.executeQuery("SELECT a.cid,b.acctype,a.name,a.description "+
					   "FROM communities a,community_access b "+
					   "WHERE b.u_id="+uid+" AND a.cid=b.cid");
	  Vector v=new Vector();
	  while(rset.next())
	    {
	      Community c=new Community();
	      c.cid=rset.getInt(1);
	      c.access_type=rset.getString(2);
	      c.name=rset.getString(3);
	      c.description=rset.getString(4);
	      v.addElement(c);
	    }
	  System.out.println(v.size());
	  Community cc[]=new Community[v.size()];
	  for(int i=0;i<v.size();i++)
	    cc[i]=(Community)v.elementAt(i);
	  return cc;
	}
      catch(Exception e)
	{
	  e.printStackTrace();
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public String		identifyUser(int uid) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt=conn.createStatement();
	  ResultSet rset=stmt.executeQuery("SELECT ulogin FROM users WHERE u_id="+uid);
	  if(rset.next())
	    {
	      return rset.getString(1);
	    }
	  else
	    return null;
	}
      catch(SQLException e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public byte[] 	getRegDta(RegKey key) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt;
	  ResultSet rset;
	  if((key.type & key.USER)!=0)
	    {
	      stmt=conn.createStatement();
	      rset=stmt.executeQuery("SELECT value FROM registry_user WHERE u_id="+
					   key.uid+" AND key='"+key.key+"'");
	      if(rset.next())
		return rset.getBytes(1);
	    }
	  if((key.type & key.COMMUNITY)!=0)
	    {
	      stmt=conn.createStatement();
	      rset=stmt.executeQuery("SELECT value FROM registry_community WHERE cid="+
					   key.cid+" AND key='"+key.key+"'");
	      if(rset.next())
		return rset.getBytes(1);
	    }
	  if((key.type & key.GLOBAL)!=0)
	    {
	      stmt=conn.createStatement();
	      rset=stmt.executeQuery("SELECT value FROM registry_global WHERE key='"+key.key+"'");
	      if(rset.next())
		return rset.getBytes(1);
	    }
	  return null;
	}
      catch(SQLException e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public Hashtable 	getRegDta(RegKey[] keys) throws java.rmi.RemoteException
    {
      Hashtable h=new Hashtable();
      for(int i=0;i<keys.length;i++)
	{
	  byte b[]=getRegDta(keys[i]);
	  if(b!=null) h.put(keys[i].key,b);
	}
      return h;
    }
  
  public void 		setRegDta(RegEntry entry) throws java.rmi.RemoteException
    {
      try
	{
	  if(entry.type==entry.USER)
	    {
	      try
		{
		  pstmt_ru_set1.setInt(1,entry.uid);
		  pstmt_ru_set1.setString(2,entry.key);
		  pstmt_ru_set1.setBytes(3,entry.value);
		  int returnval=pstmt_ru_set1.executeUpdate();
		  if(returnval<1) throw new SQLException();
		}
	      catch(SQLException e)
		{
		  try
		    {
		      pstmt_ru_set2.setInt(2,entry.uid);
		      pstmt_ru_set2.setString(3,entry.key);
		      pstmt_ru_set2.setBytes(1,entry.value);
		      int returnval=pstmt_ru_set2.executeUpdate();
		      if(returnval<1) throw new SQLException();
		    }
		  catch(SQLException ee)
		    {
		      throw new java.rmi.RemoteException("Exception in SQL",ee);
		    }
		}
	      return;
	    }
	  if(entry.type==entry.COMMUNITY)
	    {
	      try
		{
		  pstmt_rc_set1.setInt(1,entry.cid);
		  pstmt_rc_set1.setString(2,entry.key);
		  pstmt_rc_set1.setBytes(3,entry.value);
		  int returnval=pstmt_rc_set1.executeUpdate();
		  if(returnval<1) throw new SQLException();
		}
	      catch(SQLException e)
		{
		  try
		    {
		      pstmt_rc_set2.setInt(2,entry.cid);
		      pstmt_rc_set2.setString(3,entry.key);
		      pstmt_rc_set2.setBytes(1,entry.value);
		      int returnval=pstmt_rc_set2.executeUpdate();
		      if(returnval<1) throw new SQLException();
		    }
		  catch(SQLException ee)
		    {
		      throw new java.rmi.RemoteException("Exception in SQL",ee);
		    }
		}
	      return;
	    }
	  if(entry.type==entry.GLOBAL)
	    {try
		{
		  pstmt_rg_set1.setString(1,entry.key);
		  pstmt_rg_set1.setBytes(2,entry.value);
		  int returnval=pstmt_rg_set1.executeUpdate();
		  if(returnval<1) throw new SQLException();
		}
	      catch(SQLException e)
		{
		  try
		    {
		      pstmt_rg_set2.setString(2,entry.key);
		      pstmt_rg_set2.setBytes(1,entry.value);
		      int returnval=pstmt_rg_set2.executeUpdate();
		      if(returnval<1) throw new SQLException();
		    }
		  catch(SQLException ee)
		    {
		      throw new java.rmi.RemoteException("Exception in SQL",ee);
		    }
		}
	      return;
	    }
	}
      catch(Exception e)
	{
	  throw new java.rmi.RemoteException("",e);
	}
    }
  
  public void 		setRegDta(RegEntry[] entries) throws java.rmi.RemoteException
    {
      try
	{
	  conn.setAutoCommit(false);
	  try
	    {
	      for(int i=0;i<entries.length;i++)
		{
		  setRegDta(entries[i]);
		}
	    }
	  catch(Exception e)
	    {
	      conn.rollback();
	      conn.setAutoCommit(true);
	      throw e;
	    }
	  conn.commit();
	  conn.setAutoCommit(true);
	}
      catch(Exception e)
	{
	  throw new java.rmi.RemoteException("",e);
	}
    }

  public void 		setUserProperty(int uid,String key,byte[] data) throws java.rmi.RemoteException
    {
      try
	{
	  pstmt1.setInt(1,uid);
	  pstmt1.setString(2,key);
	  pstmt1.setBytes(3,data);
	  int returnval=pstmt1.executeUpdate();
	  if(returnval<1) throw new SQLException();
	}
      catch(Exception e)
	{
	  try
	    {
	      pstmt2.setInt(2,uid);
	      pstmt2.setString(3,key);
	      pstmt2.setBytes(1,data);
	      int returnval=pstmt2.executeUpdate();
	      if(returnval<1) throw new SQLException();
	    }
	  catch(SQLException ee)
	    {
	      throw new java.rmi.RemoteException("Exception in SQL",ee);
	    }
	}
    }
  
  public byte[] 	getUserProperty(int uid,String key) throws java.rmi.RemoteException
    {
      try
	{
	  Statement stmt=conn.createStatement();
	  ResultSet rset=stmt.executeQuery("SELECT value FROM user_data WHERE u_id="+
					   uid+" AND key='"+key+"'");
	  if(rset.next())
	    {
	      return rset.getBytes(1);
	    }
	  else
	    return null;
	}
      catch(SQLException e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public Hashtable 	getUserProprties(int uid,String[] keys) throws java.rmi.RemoteException
    {
      try
	{
	  String query="SELECT key,value FROM user_data WHERE u_id ="+uid+
	    " AND key IN ('"+keys[0]+"'";
	  for(int i=1;i<keys.length;i++)
	    query+=",'"+keys[i]+"'";
	  query+=")";
	  
	  Statement stmt=conn.createStatement();  
	  ResultSet rset=stmt.executeQuery(query);
	  Hashtable h=new Hashtable();
	  while(rset.next())
	    {
	      h.put(rset.getString(1),rset.getBytes(2));
	    }
	  return h;
	}
      catch(SQLException e)
	{
	  throw new java.rmi.RemoteException("Exception in SQL",e);
	}
    }
  
  public PIMTools() throws java.rmi.RemoteException,SQLException
    {
      super();
      DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
      conn=DriverManager.getConnection("jdbc:oracle:thin:@sunrise.npac.syr.edu:1521:frog","piotr","piotr");
      pstmt1=conn.prepareStatement("INSERT INTO user_data (u_id,key,value) VALUES (?,?,?)");
      pstmt2=conn.prepareStatement("UPDATE user_data SET value=? WHERE u_id=? AND key=?");
      pstmt_ru_set1=conn.prepareStatement("INSERT INTO registry_user (u_id,key,value) VALUES (?,?,?)");
      pstmt_ru_set2=conn.prepareStatement("UPDATE registry_user SET value=? WHERE u_id=? AND key=?");
      pstmt_rc_set1=conn.prepareStatement("INSERT INTO registry_community (cid,key,value) VALUES (?,?,?)");
      pstmt_rc_set2=conn.prepareStatement("UPDATE registry_community SET value=? WHERE cid=? AND key=?");
      pstmt_rg_set1=conn.prepareStatement("INSERT INTO registry_global (key,value) VALUES (?,?)");
      pstmt_rg_set2=conn.prepareStatement("UPDATE registry_global SET value=? WHERE key=?");
    }

}
















